%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% morse2text.m
% version 1.0
% developed by J. Fiene and G. Johnson
% University of Pennsylvania
% September 13, 2009
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function plaintext = morse2text(morse_string)
% outputs plaintext when given a morse code string corresponding 
% to Morse Code, where:
%   0 is an inter-character space
%   1 is a DIT
%   2 is a DAH
%   3 is an inter-word space 
% you can use Morse corresponding to the the following:
%   all 26 characters
%   all 10 digits
%   the following punctuation: !"$&'()+'-./:;=?@

ascii_offset = 31; % space is '32'

morse_codes = {
    [3]
    [2 1 2 1 2 2]
    [1 2 1 1 2 1]
    []
    [1 1 1 2 1 1 2]
    []
    [1 2 1 1 1]
    [1 2 2 2 2 1]
    [2 1 2 2 1]
    [2 1 2 2 1 2]
    []
    [1 2 1 2 1]
    [2 2 1 1 2 2]
    [2 1 1 1 1 2]
    [1 2 1 2 1 2]
    [2 1 1 2 1]
    [2 2 2 2 2]
    [1 2 2 2 2]
    [1 1 2 2 2]
    [1 1 1 2 2]
    [1 1 1 1 2]
    [1 1 1 1 1]
    [2 1 1 1 1]
    [2 2 1 1 1]
    [2 2 2 1 1]
    [2 2 2 2 1]
    [2 2 2 1 1 1]
    [2 1 2 1 2 1]
    []
    [2 1 1 1 2]
    []
    [1 1 2 2 1 1]
    [1 2 2 1 2 1]
    [1 2] 
    [2 1 1 1]
    [2 1 2 1]
    [2 1 1]
    [1]
    [1 1 2 1]
    [2 2 1]
    [1 1 1 1]
    [1 1]
    [1 2 2 2]
    [2 1 2]
    [1 2 1 1]
    [2 2]
    [2 1]
    [2 2 2]
    [1 2 2 1]
    [2 2 1 2]
    [1 2 1]
    [1 1 1]
    [2]
    [1 1 2]
    [1 1 1 2]
    [1 2 2]
    [2 1 1 2]
    [2 1 2 2]
    [2 2 1 1]
};

% translate 0123 code into plaintext
char_count = 0;
j=0;
for(i=1:length(morse_string))
    % starting code should be either 1, 2, or 3
    if(morse_string(i) ~= 0) % while it's not zero
      j = j + 1;
      character(j) = morse_string(i); % stuff it into character
    else
      char_count = char_count + 1;
      characters{char_count} = character;
      j = 0;
      character = []; % reset character
    end
end

ascii_codes = [];
for i=1:char_count % for each character
    for j=1:length(morse_codes) % check through all the morse codes
        if(length(characters{i}) == length(morse_codes{j})) % same length
            if(sum((characters{i} == morse_codes{j})==0) == 0) % same code
                ascii_codes = [ascii_codes j+ascii_offset]; % that's it!
            end
        end
    end
end

plaintext = native2unicode(ascii_codes);